#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2023/3/23 19:03
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : pdfHelper.py
# @Descr   : pdf操作
# @Software: PyCharm
import os

import comtypes.client
import fitz


# 将pdf转换为img
@staticmethod
def convert_pdf_to_img(pdf_file):
    doc = fitz.open(pdf_file)
    (file_pre_path, temp_filename) = os.path.split(pdf_file)
    (shot_name, file_ext) = os.path.splitext(temp_filename)
    img_path_list = []
    for pg in range(doc.pageCount):
        page = doc[pg]
        rotate = int(0)
        # 每个尺寸的缩放系数为3，这将为我们生成分辨率提高6倍的图像。
        zoom_x, zoom_y = 3, 3
        trans = fitz.Matrix(zoom_x, zoom_y).preRotate(rotate)
        pm = page.getPixmap(matrix=trans, alpha=False)
        img_path = os.path.join(file_pre_path, shot_name + "_" + str(pg) + ".png")
        img_path_list.append(img_path)
        pm.writePNG(img_path)
    return "&&&".join(img_path_list)


# 将word转pdf
@staticmethod
def convert_doc_to_pdf(word_path, pdf_path):
    (file_pre_path, temp_filename) = os.path.split(word_path)
    (shot_name, file_ext) = os.path.splitext(temp_filename)
    # pdf_path = os.path.join(file_pre_path, shot_name + '.pdf')
    comtypes.CoInitialize()
    word = comtypes.client.CreateObject("Word.Application")
    word.Visiable = 0  # 设置可见性，不可见
    newpdf = word.Documents.Open(word_path)
    newpdf.SaveAS(pdf_path, FileFormat=17)  # 17表示PDF格式
    newpdf.Close()
    return pdf_path





#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
# @Time    :  2022/9/28 10:36
# @Author  : chenxw
# @Email   : gisfanmachel@gmail.com
# @File    : worddir2xls.py
# @Descr   : 
# @Software: PyCharm

from docx import Document
from openpyxl import Workbook
from openpyxl.styles import Alignment, Border, borders, colors, Side

# 原始word文件和要生成的Excel文件
fn_word = 'E:\\tt.docx'
fn_excel = fn_word[:-5] + '2.xlsx'

# 创建空白Excel文件，获取第一个空白工作表
wb = Workbook()
ws = wb.worksheets[0]
# 添加表头
ws.append(['一级目录', '二级目录', '三级目录', '四级目录', '五级目录', '六级目录'])

first2, first3, first4, first5, first6 = 1, 1, 1, 1, 1
# 遍历word文件中的所有段落文本
for p in Document(fn_word).paragraphs:
    # 删除段落文本两侧的空白字符
    txt = p.text.rstrip()
    # 直接跳过空行
    if not txt:
        continue

    # 添加为一级标题
    if p.style.name == 'Heading 1':
        first2 = 1
        print(p.text)
        ws.append([txt, '', '', '', '', ''])
    # 添加为二级标题
    elif p.style.name == 'Heading 2':
        print(p.text)
        if first2 == 1:
            # 第一个二级目录，不添加新行
            # 直接和一级目录写在同一行
            # print(vgis_txt)
            list(ws.rows)[-1][1].value = txt
            first2 = first2 + 1
        elif first2 > 1:
            # 新增一行添加为二级标题
            # print(vgis_txt)
            ws.append(['', txt, '', '', '', ''])
        first3 = 1
    # 添加为三级标题
    elif p.style.name == 'Heading 3':
        print(p.text)
        if first3 == 1:
            # 第一个三级目录，不添加新行，直接写和二级在同一行
            # print(vgis_txt)
            list(ws.rows)[-1][2].value = txt
            first3 = first3 + 1
        elif first3 > 1:
            # 新增一行添加为三级目录
            print(txt)
            ws.append(['', '', txt, '', '', ''])
    # 添加为四级标题
    elif p.style.name == 'Heading 4':
        print(p.text)
        if first4 == 1:
            # 第一个四级目录，不添加新行，直接写和三级在同一行
            # print(vgis_txt)
            list(ws.rows)[-1][3].value = txt
            first4 = first4 + 1
        elif first4 > 1:
            # 新增一行添加为四级目录
            print(txt)
            ws.append(['', '', '', txt, '', ''])
    # 添加为五级标题
    elif p.style.name == 'Heading 5':
        print(p.text)
        if first5 == 1:
            # 第一个五级目录，不添加新行，直接写和四级在同一行
            # print(vgis_txt)
            list(ws.rows)[-1][4].value = txt
            first5 = first5 + 1
        elif first5 > 1:
            # 新增一行添加为五级目录
            print(txt)
            ws.append(['', '', '', '', txt, ''])
    # 添加为六级标题
    elif p.style.name == 'Heading 6':
        print(p.text)
        if first6 == 1:
            # 第一个六级目录，不添加新行，直接写和五级在同一行
            # print(vgis_txt)
            list(ws.rows)[-1][5].value = txt
            first6 = first6 + 1
        elif first6 > 1:
            # 新增一行添加为六级目录
            print(txt)
            ws.append(['', '', '', '', '', txt])

wb.save(fn_excel)

# 设置单元格边框
side = Side(border_style=borders.BORDER_THICK,
            color=colors.BLACK)
for row in ws.rows:
    for cell in row:
        cell.border = Border(top=side, bottom=side,
                             left=side, right=side)
start1, start2 = 1, 1
for index, row in enumerate(ws.rows, start=1):
    # 合并第一列的指定单元格，水平左对齐，垂直居中对齐
    if row[0].value:
        if index > start1:
            ws.merge_cells(f'A{start1}:A{index - 1}')
            ws[f'A{start1}'].alignment = Alignment(horizontal='left',
                                                   vertical='center')
        start1 = index
    # 合并第二列的指定单元格，水平左对齐，垂直居中对齐
    if row[1].value:
        if index > start2:
            ws.merge_cells(f'B{start2}:B{index - 1}')
            ws[f'B{start2}'].alignment = Alignment(horizontal='left',
                                                   vertical='center')
        start2 = index
if index > start1:
    ws.merge_cells(f'A{start1}:A{index}')
    ws[f'A{start1}'].alignment = Alignment(horizontal='left',
                                           vertical='center')

if index > start2:
    ws.merge_cells(f'B{start2}:B{index}')
    ws[f'B{start2}'].alignment = Alignment(horizontal='left',
                                           vertical='center')

wb.save(fn_excel)



"""
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Project :pythonCodeSnippet
@File    :wordTools.py
@IDE     :PyCharm
@Author  :chenxw
@Date    :2023/9/22 14:51
@Descr:
"""
import os

from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from docx.oxml.ns import qn
from docx.shared import Cm
from docx.shared import Pt
import win32com.client
from pathlib import Path
import pythoncom

class WordHelper:

    @staticmethod
    # 需要安装wps
    def wps2docx(wps_file_path, save_path):
        pythoncom.CoInitialize()
        # 使用 WPS 打开文档
        wps = win32com.client.Dispatch("Kwps.Application")
        wps.Visible = False  # 可选，设置是否显示 WPS 界面
        doc = wps.Documents.Open(os.path.abspath(wps_file_path), ReadOnly=1)
        if wps_file_path == save_path:
            os.remove(wps_file_path)
        # 另存为 docx 文件
        doc.SaveAs(os.path.abspath(save_path), 12)  # 12 表示 docx 格式

        # 关闭文档和 WPS 应用
        doc.Close()
        wps.Quit()
        pythoncom.CoUninitialize()

    @staticmethod
    # 需要安装wps或office
    def doc2docx(input_filepath, output_filepath, keep_active=True):
        pythoncom.CoInitialize()
        input_filepath = Path(input_filepath).resolve()
        output_filepath = Path(output_filepath).resolve()
        word_app = win32com.client.Dispatch("Word.Application")
        doc = word_app.Documents.Open(str(input_filepath))
        try:
            doc.SaveAs2(str(output_filepath), FileFormat=16)
            doc.Close(0)
        except:
            doc.Close(0)

        if not keep_active:
            word_app.Quit()
        pythoncom.CoUninitialize()
    @staticmethod
    # 替换报告单里段落文字:{word}
    # python替换word中的书签变量{word}， 在读取{}时，获取runs不知道为什么会有些连在一起的文字会被拆掉（即使样式一样）
    # 解决：修改{word}里变量名，或在变量名称里加个},比如{word}}
    # obj={}
    # obj["{name}"]=""
    # obj["{age}"]=""
    def replace_txt_in_word(document, obj, logger):
        try:

            # 循环所有段落
            for paragraph in document.paragraphs:
                # run是根据样式来分的，如果文字在一起，但是不同样式，也会分成两个run
                for run in paragraph.runs:
                    # print(run.text)
                    for key, value in obj.items():
                        key = "{}".format(key)
                        if key in run.text:
                            run.text = run.text.replace(key, "{}".format(value))
                            # break
            # 循环所有表格
            for table in document.tables:
                for row in table.rows:
                    for cell in row.cells:
                        cell_text = cell.text
                        # print(cell_text)
                        for key, value in obj.items():
                            key = "{}".format(key)
                            if key in cell_text:
                                for paragraph in cell.paragraphs:
                                    if key in paragraph.text:
                                        for run in paragraph.runs:
                                            run.text = run.text.replace(key, "{}".format(value))


        except Exception as exp:
            logger.error("替换word里的文本失败：" + str(exp))
            # 输出异常信息
            logger.error(exp)
            logger.error(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            logger.error(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

    @staticmethod
    # 填充报告里的表格
    # obj_list:[{obj}],obj的key与表格的列名一致
    # obj = {}
    # obj["序号"] = ""
    # obj["姓名"] = ""
    # obj["身份证"] = ""
    def fill_table_in_word(table, obj_list, logger, show_max_rows):
        try:
            all_data_list = obj_list
            if len(all_data_list) > 0:

                rows = table.rows
                headCells = rows[0].cells  # 获取头
                for i in range(len(all_data_list)):
                    print("写入" + str(i))
                    # 只输出前***条,避免行数过多，把表格撑爆
                    if i >= show_max_rows:
                        break
                    rowData = all_data_list[i]
                    # 插入表格行数据
                    if i == 0:
                        rowCells_record = table.rows[1].cells
                    else:
                        rowCells_record = table.add_row().cells
                    for index in range(len(headCells)):
                        head = headCells[index]
                        # rowCells_record[index].text = str(rowData.get(head.text))
                        run = rowCells_record[index].paragraphs[0].add_run(str(rowData.get(head.text)))
                        run.font.name = '宋体'
                        run.font.size = Pt(10)
                        r = run._element
                        r.rPr.rFonts.set(qn('w:eastAsia'), '宋体')
                        # 设置表格的对齐方式
                        rowCells_record[index].paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER
                        rowCells_record[index].vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER

        except Exception as exp:
            logger.error("填充word里的表格失败：" + str(exp))
            # 打印输出异常信息
            logger.error(exp)
            logger.error(exp.__traceback__.tb_frame.f_globals["__file__"])  # 发生异常所在的文件
            logger.error(exp.__traceback__.tb_lineno)  # 发生异常所在的行数

    @staticmethod
    # 替换报告里的的图片
    def replace_image_in_word(document, obj, input_img_path):

        # 循环所有表格
        for table in document.tables:
            for row in table.rows:
                for cell in row.cells:
                    cell_text = cell.text
                    # print(cell_text)
                    for key, value in obj.items():
                        key = "{}".format(key)
                        if key in cell_text:
                            for paragraph in cell.paragraphs:
                                # if key in paragraph.text:
                                #     # 把占位符去掉
                                #     paragraph.text = paragraph.text.replace('{' + key + '}', '')
                                #     run = paragraph.add_run('')
                                #     # 添加图片并指定大小
                                #     if obj[key] != "":
                                #         img_path = CommonHelper.remove_url_head(obj[key])
                                #         input_img = str(settings.BASE_DIR) + "/my_app" + img_path
                                #         print(input_img)
                                #         # img = Image.open(input_img)
                                #         run.add_picture(input_img, height=Cm(1.14))
                                if key in paragraph.text:
                                    has_replaced = False
                                    for run in paragraph.runs:
                                        run.clear()
                                        # 增加转行，使得下面插入图片往下走
                                        run.add_break()
                                        run.add_break()
                                        if not has_replaced:
                                            if obj[key] != "":
                                                print(input_img_path)
                                                if os.path.exists(input_img_path):
                                                    run.add_picture(input_img_path, width=Cm(2.69))
                                                has_replaced = True



"""
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Project :vgis_python_package
@File    :pptTools.py
@IDE     :PyCharm
@Author  :chenxw
@Date    :2024/7/31 14:47
@Descr:
"""
import os

import win32com.client
from pathlib import Path
import pythoncom


class PptHelper:
    @staticmethod
    # 需要安装了wps或office
    def ppt2pptx(input_filepath, output_filepath, keep_active=True):
        pythoncom.CoInitialize()
        input_filepath = Path(input_filepath).resolve()
        output_filepath = Path(output_filepath).resolve()
        ppt_app = win32com.client.Dispatch("Powerpoint.Application")
        presentation = ppt_app.Presentations.Open(str(input_filepath), 0, 0, 0)
        try:
            presentation.SaveAs(str(output_filepath), FileFormat=24)
            presentation.Close()
        except:
            presentation.Close()

        if not keep_active:
            ppt_app.Quit()
        pythoncom.CoUninitialize()




"""
===================================
#!/usr/bin/python3.9
# -*- coding: utf-8 -*-
@Author: chenxw
@Email : gisfanmachel@gmail.com
@File: excelTools.py
@Date: Create in 2021/1/28 11:53
@Description: excel操作类
@ Software: PyCharm
===================================
"""
import os
import string

import openpyxl
import pandas as pd
from openpyxl.styles import Alignment
from openpyxl.styles import Border, Side

from vgis_utils.vgis_file.fileTools import FileHelper
from vgis_utils.vgis_list.listTools import ListHelper

import win32com.client
from pathlib import Path
import pythoncom

class ExcelHelper:

    @staticmethod
    # 需要安装了wps或office
    def xls2xlsx(input_filepath, output_filepath, keep_active=True):
        pythoncom.CoUninitialize()
        input_filepath = Path(input_filepath).resolve()
        output_filepath = Path(output_filepath).resolve()
        excel_app = win32com.client.Dispatch("Excel.Application")
        sheet = excel_app.Workbooks.Open(str(input_filepath))
        try:
            sheet.SaveAs(str(output_filepath), FileFormat=51)
            sheet.Close(0)
        except:
            sheet.Close(0)

        if not keep_active:
            excel_app.Quit()
        pythoncom.CoUninitialize()

    # 判断excel表里是否有某列
    @staticmethod
    def is_has_field_in_excel(excel_obj, field_name):
        is_find = False
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip() == field_name.strip():
                is_find = True
                break
        return is_find

    # 判断excel表里是否有某列(列名去中间空格）
    @staticmethod
    def is_has_field_of_remove_space_in_excel(excel_obj, field_name):
        is_find = False
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip().replace(" ", "") == field_name.strip().replace(" ", ""):
                is_find = True
                break
        return is_find

    @staticmethod
    # 通过excel字段名获取字段索引，若没找到，则为-1
    def get_field_index_by_name_in_excel(excel_obj, field_name):
        field_index_need = -1
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip() == field_name.strip():
                field_index_need = field_index
                break
        return field_index_need

    @staticmethod
    # 通过excel字段名（去掉中间空格）获取字段索引，若没找到，则为-1
    def get_field_index_by_name_of_remove_space_in_excel(excel_obj, field_name):
        field_index_need = -1
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip().replace(" ", "") == field_name.strip().replace(" ", ""):
                field_index_need = field_index
                break
        return field_index_need

    @staticmethod
    # 通过excel字段名（去掉中间空格和换行）获取字段索引，若没找到，则为-1
    def get_field_index_by_name_of_remove_space_and_breakline_in_excel(excel_obj, field_name):
        field_index_need = -1
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip().replace(" ", "").replace("\n", "") == field_name.strip().replace(" ",
                                                                                                                 "").replace(
                "\n", ""):
                field_index_need = field_index
                break
        return field_index_need

    @staticmethod
    # 通过excel字段名和行号获取字段值
    def get_field_value_by_name_in_excel(excel_obj, row_index, field_name):
        field_index_need = -1
        column_list = excel_obj.columns.values
        for field_index in range(len(column_list)):
            if column_list[field_index].strip() == field_name.strip():
                field_index_need = field_index
                break
        row_values = excel_obj.values[row_index]
        return row_values[field_index_need]

    @staticmethod
    # 获取excel的字段个数
    def get_field_count_in_excel(excel_obj):
        column_list = excel_obj.columns.values
        return len(column_list)

    @staticmethod
    # 为excel增加一列
    def add_field_in_excel(excel_obj, field_name):
        excel_obj[field_name] = None

    @staticmethod
    # 读取excel表内容指定行数据
    def read_excel_data_values_by_row(excel_obj, row_num):
        for row_index in range(len(excel_obj)):
            # excel内容从第二行开始
            if row_index == row_num - 1:
                result_row_values = excel_obj.values[row_index]
                return result_row_values

    @staticmethod
    #  读取excel表字段
    def read_excel_data_columns(excel_obj):
        result_row_values = excel_obj.columns
        return result_row_values

    @staticmethod
    # 构建新excel结果数据
    def build_data_excel(result_excel_path, all_data_list):
        file_dir_exists = os.path.exists(os.path.dirname(result_excel_path))
        if file_dir_exists is False:
            os.makedirs(os.path.dirname(result_excel_path))
        if os.path.exists(result_excel_path):
            os.remove(result_excel_path)
        if len(all_data_list) > 0:
            with pd.ExcelWriter(result_excel_path) as writer:
                for i in range(len(all_data_list)):
                    each_data_list = all_data_list[i]
                    if len(each_data_list) > 0:
                        df = pd.DataFrame(each_data_list,
                                          columns=ListHelper.get_key_name_str(each_data_list[0]))
                        sheet_name_str = "sheet" + str(i + 1)
                        df.to_excel(writer, sheet_name=sheet_name_str, index=False, startrow=0, encoding="utf_8_sig")
                # writer.save()
                # writer.close()
        else:
            # 创建空白excel
            fd = open(result_excel_path, 'w')
            fd.close()

    # 更新excel数据
    # 模板文件路径excel_template_path
    # 新文件路径 excel_target_path
    # 数据所在的sheet：sheet_name
    # 更新数据data_list:[{},{},{}]
    # 更新值开始行 value_start_row
    # 更新值开始列 value_start_col
    # 设置border结束列，border_end_col 这个大于等于更新值的结束列，因为excel可能有些其他列存在
    @staticmethod
    def update_excel(excel_template_path, excel_target_path, sheet_name, data_list, value_start_row, value_start_col,
                     border_end_col):
        FileHelper.copy_file(excel_template_path, excel_target_path)
        wb = openpyxl.load_workbook(excel_target_path)  # 打开excel工作簿
        # ws = wb.active  # 获取活跃sheet
        # ws = wb[wb.sheetnames[0]] #获取第一个sheet
        ws = wb[sheet_name]  # 获取指定名称的sheet
        # ws.cell(2, 1).value = 4
        # ws.cell(2, 2).value = 5
        # ws.cell(2, 3).value = 6
        # ws.cell(3, 1).value = 7
        # ws.cell(3, 2).value = 8
        # ws.cell(3, 3).value = 9
        data_row_index = 0
        for each_data in data_list:
            data_col_index = 0
            for key, value in each_data.items():
                ws.cell(value_start_row + data_row_index, value_start_col + data_col_index).value = value
                data_col_index += 1
            data_row_index += 1

        # border style
        border = Border(left=Side(border_style='thin', color='000000'), right=Side(border_style='thin', color='000000'),
                        top=Side(border_style='thin', color='000000'), bottom=Side(border_style='thin', color='000000'))
        # cell format , horizontal: 水平对齐；vertical：中心对齐；wrap_text：自动换行、
        align = Alignment(horizontal='center', vertical='center', wrap_text=True)

        value_end_row = value_start_row + len(data_list) - 1
        # set cell border
        for row in range(value_start_row, value_end_row + 1):
            for col in range(value_start_col, border_end_col + 1):
                ws.cell(row, col).border = border
                ws.cell(row, col).alignment = align

        wb.save(excel_target_path)  # 保存

    @staticmethod
    # 根据Excel列索引值生成列字母名
    def get_column_name(column_index):
        ret = ''
        ci = column_index - 1
        index = ci // 26
        if index > 0:
            ret += ExcelHelper.get_column_name(index)
        ret += string.ascii_uppercase[ci % 26]
        return ret

    @staticmethod
    def get_field_name_by_field_index(excel_obj, field_index):
        field_name_need = ""
        column_list = excel_obj.columns.values
        for tmp_index in range(len(column_list)):
            if field_index == tmp_index:
                field_name_need = column_list[field_index].strip().replace(" ", "")
                break
        return field_name_need


if __name__ == '__main__':
    data_list = []
    obj = {}
    obj["序号"] = 1
    obj['批改标志'] = "增加"
    obj["分组组别"] = "2组"
    obj['被保险人名称'] = "中核1"
    obj['证件类型'] = '居民身份证'
    obj['证件号码'] = "23325252352"
    obj['性别'] = "男"
    obj['年龄'] = 25
    obj['出生日期\n(YYYY-MM-DD)'] = "2034-45-23"
    obj['职业代码人员类别'] = "100023"
    data_list.append(obj)
    obj = {}
    obj["序号"] = 2
    obj['批改标志'] = "减少"
    obj["分组组别"] = "3组"
    obj['被保险人名称'] = "中核3"
    obj['证件类型'] = '居民身份证'
    obj['证件号码'] = "54544662525235332"
    obj['性别'] = "女"
    obj['年龄'] = 32
    obj['出生日期\n(YYYY-MM-DD)'] = "2294-45-23"
    obj['职业代码人员类别'] = "100029"
    data_list.append(obj)

    excel_template_path = "E:\\系统开发\\中核保险辅助出单\\zwbx_fzcd_service\\my_project\\my_app\\zhbxfzcd_static\\template\\POLICY_PEOPLE_INFORMATION_TEMPLATE.xlsx"
    excel_target_path = "E:\\系统开发\\中核保险辅助出单\\zwbx_fzcd_service\\my_project\\my_app\\zhbxfzcd_static\\template\\POLICY_PEOPLE_INFORMATION_NEW8.xlsx"
    ExcelHelper.update_excel(excel_template_path, excel_target_path, "人员清单", data_list, 2, 1, 10)

    excel_template_path = "E:\\系统开发\\中核保险辅助出单\\zwbx_fzcd_service\\my_project\\my_app\\zhbxfzcd_static\\template\\ENDORSEMENT_PEOPLE_INFORMATION_TEMPLATE.xlsx"
    excel_target_path = "E:\\系统开发\\中核保险辅助出单\\zwbx_fzcd_service\\my_project\\my_app\\zhbxfzcd_static\\template\\ENDORSEMENT_PEOPLE_INFORMATION_NEW7.xlsx"
    ExcelHelper.update_excel(excel_template_path, excel_target_path, "人员清单", data_list, 5, 1, 83)
